@@ -2,9 +2,9 @@
 ### ====================================================================
 ###  @UNIX-shell-file{
 ###     author          = "Nelson H. F. Beebe",
-###     version         = "0.07",
-###     date            = "25 September 1994",
-###     time            = "13:12:34 MDT",
+###     version         = "0.09",
+###     date            = "17 January 1996",
+###     time            = "07:54:42 MST",
 ###     filename        = "bibsort.sh",
 ###     address         = "Center for Scientific Computing
 ###                        Department of Mathematics
@@ -13,7 +13,7 @@
 ###                        USA",
 ###     telephone       = "+1 801 581 5254",
 ###     FAX             = "+1 801 581 4148",
-###     checksum        = "27435 515 2349 20334",
+###     checksum        = "22155 646 2840 24287",
 ###     email           = "beebe@math.utah.edu (Internet)",
 ###     codetable       = "ISO/ASCII",
 ###     keywords        = "bibliography, sorting, BibTeX",
@@ -25,25 +25,48 @@
 ###                        switches, as described below.
 ###
 ###                        Usage:
-###                              bibsort [-byvolume or -byyear] \
+###                              bibsort [-byday or -byvolume or -byyear] \
 ###                                      [optional sort(1) switches] \
 ###                                      bibfile(s) >outfile
 ###                        or
-###                              bibsort [-byvolume or -byyear] \
+###                              bibsort [-byday or -byvolume or -byyear] \
 ###                                      [optional sort(1) switches] \
 ###                                      <infile >outfile
 ###
 ###                        Bibliography items are normally sorted in
 ###                        ascending order by citation label.  However,
-###                        this sorting is modified if a -byvolume or
-###                        -byyear switch, or certain other sort(1)
-###                        switches, are specified.
+###                        this sorting is modified if a -byday,
+###                        -byvolume, or -byyear switch, or certain
+###                        other sort(1) switches, are specified.
 ###
-###                        -byvolume and -byyear may be abbreviated to
-###                        unique prefixes -byv and -byy, if desired.
-###                        Switches may appear in any order; however, if
-###                        both -byvolume and -byyear are specified,
-###                        only the last one is used.
+###                        -byday, -byvolume and -byyear may be
+###                        abbreviated to unique prefixes -byd, -byv and
+###                        -byy, if desired.  Switches may appear in any
+###                        order; however, if more than one -byxxx
+###                        switch is specified, only the last one is
+###                        used.
+###
+###                        The -byday switch is intended for use with
+###                        bibliographies of publications containing
+###                        day, month, and year data, such as technical
+###                        reports, newspapers, and magazines.  It
+###                        causes entries to be sorted by year, month,
+###                        day, and citation label, so that the entries
+###                        appear in their original publication order.
+###
+###                        With -byday sorting, a day keyword is
+###                        recognized (it will be standard in BibTeX
+###                        1.0), but for backward compability, month
+###                        entries of the form
+###
+###                        <quote>daynumber <quote> # monthname
+###                        <quote>daynumber~<quote> # monthname
+###                        monthname # <quote>daynumber <quote>
+###                        monthname # <quote>daynumber~<quote>
+###
+###                        are also recognized.  If a day number is not
+###                        available, 99 is assumed, which will sort the
+###                        entry after others that have day values.
 ###
 ###                        The -byvolume switch is intended for use with
 ###                        bibliographies of single journals.  It causes
@@ -204,6 +227,7 @@
 ########################################################################
 
 # Assign default initial values
+BYDAY=0
 BYVOLUME=0
 BYYEAR=0
 FILES=
@@ -215,16 +239,31 @@ SORTFLAGS=
 while [ $# -gt -0 ]
 do
 	case $1 in
-	-byy*)
-		BYYEAR=1
+	-byd*)
+		BYDAY=1
 		BYVOLUME=0
+		BYYEAR=0
+ 		SORTFLAGS=
+		;;
+	-byy*)
+		BYDAY=0
+		BYVOLUME=0
+		BYYEAR=1
  		SORTFLAGS=
 		;;
 	-byv*)
+		BYDAY=0
 		BYVOLUME=1
 		BYYEAR=0
-		# key = <group><C-k><journal><C-k><volume><C-k><number-1><C-k><number-2><C-k><pages><C-k><citation-label>
-		SORTFLAGS="-t +0 -1 +1 -2 +2n -3 +3n -4 +4n -5 +5n -6 +6 -7"
+		# key = <group>
+		#	<C-k><journal>
+		#	<C-k><year>
+		#	<C-k><volume>
+		#	<C-k><number-1>
+		#	<C-k><number-2>
+		#	<C-k><pages>
+		#	<C-k><citation-label>
+		SORTFLAGS="-t +0 -1 +1 -2 +2n -3 +3n -4 +4n -5 +5n -6 +6n -7 +7 -8"
 		;;
 	-*)			# all other switches are passed to sort
 		OTHERSORTFLAGS="$OTHERSORTFLAGS $1"
@@ -237,22 +276,7 @@ do
 done
 
 # We store the awk program as a (large) string constant
-PROGRAM='BEGIN {
-	prefix_header		= "\001"
-	prefix_preamble		= "\002"
-	prefix_string		= "\003"
-	prefix_normal_entry	= "\004"
-	prefix_cross_referenced	= "\177"
-        sort_prefix		= "\005"
-        hidden_newline		= "\006"
-	visible_newline		= "\007"
-	unknown_value		= "\377" 	# such entries sort last
-	volume_key_separator	= "\013"	# C-k character
-
-        sort_key		= sort_prefix "%%SORTKEY:"
-
-        print sort_key prefix_header hidden_newline
-}
+PROGRAM='BEGIN { initialize() }
 
 /^[ \t]*@[ \t]*[Pp][Rr][Ee][Aa][Mm][Bb][Ll][Ee][ \t]*{/ {
 	trim()
@@ -358,12 +382,99 @@ function collect_braced_item( count,item)
     return item
 }
 
+function day_key(item,citation_key, day,month,n,parts,year)
+{
+    day   = value(item,citation_key,"day[ \t]*=[ \t]*")
+    month = value(item,citation_key,"month[ \t]*=[ \t]*")
+    year  = value(item,citation_key,"year[ \t]*=[ \t]*")
+
+    # Expect month values like this:
+    #	jan
+    #	January
+    #	jan # " 10"
+    #	jan # "~10"
+    #	"10 " # jan
+    #	"10~" # jan
+    #	"10 January"
+    #	"10~January"
+    #	"January 10"
+    #	"January~10"
+    #	jan # { 10}
+    #	jan # {~10}
+    #	{10 } # jan
+    #	{10~} # jan
+    #	{10 January}
+    #	{10~January}
+    #	{January 10}
+    #	{January~10}
+
+    gsub(/[{}\t\#\"~]/," ",month) # remove delimiters, ties, concatenation
+    gsub(/[ ]+/," ",month)	# squeeze multiple spaces to single ones
+    n = split(month,parts," ")
+
+    #### print "DEBUG: " citation_key ": year=[" year "] month=[" month \
+    #### "] n=" n " parts[1]=[" parts[1] "] parts[2]=[" parts[2] "]" >"/dev/tty"
+
+    if (parts[1] ~ /^[a-zA-Z]+$/)
+	month = month_number[substr(tolower(parts[1]),1,3)]
+    else if (parts[2] ~ /^[a-zA-Z]+$/)
+	month = month_number[substr(tolower(parts[2]),1,3)]
+    else
+	month = "99"
+
+    if (day == unknown_value)	# then expect day in month value
+    {
+	if (parts[1] ~ /^[0-9]+$/)
+	    day = parts[1]
+	else if (parts[2] ~ /^[0-9]+$/)
+	    day = parts[2]
+	else
+	    day = "99"
+    }
+
+    if (year !~ /^[12][0-9][0-9x][0-9x]$/)
+	year = "9999"
+    return ( year sprintf("%02d",month) sprintf("%02d",day) )
+}
+
+
 function error(message)
 {		# print a message and terminate with failing exit code
     warning(message)
     exit(1)
 }
 
+function initialize()
+{
+    prefix_header		= "\001"
+    prefix_preamble		= "\002"
+    prefix_string		= "\003"
+    prefix_normal_entry		= "\004"
+    prefix_cross_referenced	= "\177"
+    sort_prefix			= "\005"
+    hidden_newline		= "\006"
+    visible_newline		= "\007"
+    unknown_value		= "\377" 	# such entries sort last
+    volume_key_separator	= "\013"	# C-k character
+
+    sort_key			= sort_prefix "%%SORTKEY:"
+
+    print sort_key prefix_header hidden_newline
+
+    month_number["jan"]		= 1
+    month_number["feb"]		= 2
+    month_number["mar"]		= 3
+    month_number["apr"]		= 4
+    month_number["may"]		= 5
+    month_number["jun"]		= 6
+    month_number["jul"]		= 7
+    month_number["aug"]		= 8
+    month_number["sep"]		= 9
+    month_number["oct"]		= 10
+    month_number["nov"]		= 11
+    month_number["dec"]		= 12
+}
+
 function print_braced_item(count)
 {
     # Starting with the current contents of $0, print lines until we
@@ -387,12 +498,14 @@ function print_item(prefix,citation_key,item, extra_key,v)
     if (citation_key in cross_referenced_item) # change prefix if this item
 	prefix = prefix_cross_referenced # was cross-referenced earlier
 
-    if (BYYEAR)
+    if (BYDAY)
+	extra_key = day_key(item,citation_key)
+    else if (BYVOLUME)
+	extra_key = volume_key(item,citation_key)
+    else if (BYYEAR)
 	extra_key = (match(item,/:[12][0-9][0-9x][0-9x]:/)) ? \
 	    substr(item,RSTART,RLENGTH) : \
 	    (":" value(item,citation_key,"year[ \t]*=[ \t]*") ":")
-    else if (BYVOLUME)
-	extra_key = volume_key(item,citation_key)
     else
 	extra_key = ""
 
@@ -423,27 +536,27 @@ function trim()
     sub(/[ \t]+$/,"")
 }
 
-function value(item,citation_key,keyword_pattern, s,v)
+function value(item,citation_key,keyword_pattern, n,s,v)
 {
     match(item,keyword_pattern)
     ### print "DEBUG: value() [" substr(item,RSTART,RLENGTH) "] [" item "]\n\n" >"/dev/tty"
     if (substr(item,RSTART+RLENGTH,1) == "\"") # have key = "value"
     {
 	s = substr(item,RSTART+RLENGTH)
-	match(s,/["][^"]+["]/)
+	match(s,/["][^"]*["]/)
 	v = (RLENGTH > 2) ? substr(s,RSTART+1,RLENGTH-2) : unknown_value
     }
     else if (substr(item,RSTART+RLENGTH,1) == "{") # have key = {value}
     {
 	s = substr(item,RSTART+RLENGTH)
-	match(s,/{[^}]+}/)
+	match(s,/{[^}]*}/)
 	v = (RLENGTH > 2) ? substr(s,RSTART+1,RLENGTH-2) : unknown_value
     }
     else if (substr(item,RSTART+RLENGTH,1) ~ /[0-9]/) # have key = number,
     {
 	s = substr(item,RSTART+RLENGTH)
 	match(s,/[^,]+,/)
-	 v = (RLENGTH > 1) ? substr(s,RSTART,RLENGTH-1) : unknown_value
+	v = (RLENGTH > 1) ? substr(s,RSTART,RLENGTH-1) : unknown_value
     }
     else if (substr(item,RSTART+RLENGTH,1) ~ /[A-Za-z]/) # have key = abbrev,
     {
@@ -454,6 +567,23 @@ function value(item,citation_key,keyword_pattern, s,v)
     else			# unexpected pattern
 	v = unknown_value
 
+    # For -byday sorting, we also need to return any immediately
+    # following concatenated string in a month value.  We just grab
+    # text up to the next newline or comma, which is not rigorous, but
+    # adequate for our purposes.
+    if (BYDAY && \
+	(v != unknown_value) && \
+	(substr(keyword_pattern,1,5) == "month"))
+    {
+	s = substr(s,RSTART+RLENGTH)
+	if (match(s,/[ \t\n]*\#/) && (RSTART == 1))
+	{
+	    s = substr(s,RSTART+RLENGTH)
+	    match(s,/[^\n,]+/)
+	    v = v " # " substr(s,RSTART,RLENGTH)
+	}
+    }
+
     if ((index(v,unknown_value) > 0) &&
 	(index(keyword_pattern,"crossref") == 0))
     {				# warn about missing values
@@ -471,6 +601,7 @@ function volume_key(item,citation_key, n,parts)
     return ( \
 	volume_key_separator \
 	value(item,citation_key,"journal[ \t]*=[ \t]*") volume_key_separator \
+	value(item,citation_key,"year[ \t]*=[ \t]*") volume_key_separator \
 	value(item,citation_key,"volume[ \t]*=[ \t]*") volume_key_separator \
 	value(item,citation_key,"number[ \t]*=[ \t]*") volume_key_separator \
 	parts[1] volume_key_separator  \
@@ -506,7 +637,7 @@ function warning(message)
 #
 # Finally, here is the pipeline that does all of the work:
 
-nawk "$PROGRAM" BYVOLUME=$BYVOLUME BYYEAR=$BYYEAR $FILES | \
+gawk "$PROGRAM" BYDAY=$BYDAY BYVOLUME=$BYVOLUME BYYEAR=$BYYEAR $FILES | \
         tr '\012\005' '\007\012' | \
         sort -f $SORTFLAGS $OTHERSORTFLAGS | \
         tr '\007\012' '\012\005' | \
